commit 8d1e7635e114a450dcc9a3bf2f55b777112dc88c
Author: Mahendra Singh Thalor <mahi6run@gmail.com>
Date:   Fri Jan 3 20:01:19 2020 +0530

    Enable parallel vaccum if parallel option is not given
    
    If user hasn't selected parallel option, then also enable parallel
    vacuum and decide degree of parallel vacuum based on indexes.  If
    user requested zero as degree, then disable parallel vacuum.

diff --git a/src/backend/commands/vacuum.c b/src/backend/commands/vacuum.c
index 8f60fef..f730db1 100644
--- a/src/backend/commands/vacuum.c
+++ b/src/backend/commands/vacuum.c
@@ -106,6 +106,7 @@ ExecVacuum(ParseState *pstate, VacuumStmt *vacstmt, bool isTopLevel)
 	bool		full = false;
 	bool		disable_page_skipping = false;
 	ListCell   *lc;
+	bool		disable_parallel_vacuum = false;
 
 	/* Set default value */
 	params.index_cleanup = VACOPT_TERNARY_DEFAULT;
@@ -157,12 +158,22 @@ ExecVacuum(ParseState *pstate, VacuumStmt *vacstmt, bool isTopLevel)
 			else
 			{
 				params.nworkers = defGetInt32(opt);
-				if (params.nworkers < 1 || params.nworkers > MAX_PARALLEL_WORKER_LIMIT)
+				if (params.nworkers < 0 || params.nworkers > MAX_PARALLEL_WORKER_LIMIT)
 					ereport(ERROR,
 							(errcode(ERRCODE_SYNTAX_ERROR),
-							 errmsg("parallel vacuum degree must be between 1 and %d",
+							 errmsg("parallel vacuum degree must be between 0 and %d",
 									MAX_PARALLEL_WORKER_LIMIT),
 							 parser_errposition(pstate, opt->location)));
+
+				/*
+				 * If parallel degree is given as zero, it means user want to
+				 * disable parallel vacuum.
+				 */
+				if (params.nworkers == 0)
+				{
+					disable_parallel_vacuum = true;
+					params.nworkers = -1;
+				}
 			}
 		}
 		else
@@ -213,6 +224,16 @@ ExecVacuum(ParseState *pstate, VacuumStmt *vacstmt, bool isTopLevel)
 				 errmsg("cannot specify FULL option with PARALLEL option")));
 
 	/*
+	 * If user has not requested parallel vacuum, and this is not full vacuum,
+	 * so by default, we will enable parallel vacuum and will set degree as
+	 * zero.  Later, based on indexes and size of indexes, we will decide
+	 * degree of parallel vacuum.
+	 */
+	if (!(params.options & VACOPT_FULL) && params.nworkers == -1 &&
+		!disable_parallel_vacuum)
+		params.nworkers = 0;
+
+	/*
 	 * All freeze ages are zero if the FREEZE option is given; otherwise pass
 	 * them as -1 which means to use the default values.
 	 */
diff --git a/src/test/regress/expected/vacuum.out b/src/test/regress/expected/vacuum.out
index 07c7b88..7e33955 100644
--- a/src/test/regress/expected/vacuum.out
+++ b/src/test/regress/expected/vacuum.out
@@ -107,9 +107,9 @@ VACUUM (PARALLEL 2) pvactst;
 -- VACUUM invokes parallel bulk-deletion
 UPDATE pvactst SET i = i WHERE i < 1000;
 VACUUM (PARALLEL 2) pvactst;
-VACUUM (PARALLEL 0) pvactst; -- error
-ERROR:  parallel vacuum degree must be between 1 and 1024
-LINE 1: VACUUM (PARALLEL 0) pvactst;
+VACUUM (PARALLEL -1) pvactst; -- error
+ERROR:  parallel vacuum degree must be between 0 and 1024
+LINE 1: VACUUM (PARALLEL -1) pvactst;
                 ^
 VACUUM (PARALLEL 2, INDEX_CLEANUP FALSE) pvactst;
 VACUUM (PARALLEL 2, FULL TRUE) pvactst; -- error, cannot use both PARALLEL and FULL
diff --git a/src/test/regress/sql/vacuum.sql b/src/test/regress/sql/vacuum.sql
index 6227ab9..ad7ff4e 100644
--- a/src/test/regress/sql/vacuum.sql
+++ b/src/test/regress/sql/vacuum.sql
@@ -92,8 +92,7 @@ VACUUM (PARALLEL 2) pvactst;
 -- VACUUM invokes parallel bulk-deletion
 UPDATE pvactst SET i = i WHERE i < 1000;
 VACUUM (PARALLEL 2) pvactst;
-
-VACUUM (PARALLEL 0) pvactst; -- error
+VACUUM (PARALLEL -1) pvactst; -- error
 VACUUM (PARALLEL 2, INDEX_CLEANUP FALSE) pvactst;
 VACUUM (PARALLEL 2, FULL TRUE) pvactst; -- error, cannot use both PARALLEL and FULL
 CREATE TEMPORARY TABLE tmp (a int PRIMARY KEY);
